﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Net.Sockets;
using System.Threading;
using System.IO.Ports;



namespace SSPA_Remote
{
    public partial class Form1 : Form
    {
        private System.Windows.Forms.Timer timer_HRDcom;
        private System.Windows.Forms.Timer timer_PAcom;
        Thread thread_HRDcom, thread_PAcom;

        public static Boolean hrdEnable = false;
        public static string paComPort = "COM1";
        public static string hrdIP = "172.0.0.1";
        public static string iniPath = System.Environment.GetFolderPath(System.Environment.SpecialFolder.LocalApplicationData) + "\\SSPA.ini";
        public static string hrdRadioID = "";
        public static int hrdFrequency = 0;
        SerialPort m_SerialPort;
        public string serialRCV;
        
        TcpClient tcpClient;
        Stream tcpStream;
        const int maxBuffer = 2000;

        struct PaStatus
        {
            public int status;
            public int antsw;
            public string temp1;
            public string temp2;
            public string voltage;
            public string current;
            public string powerDC;
            public string eta;
            public string powerAnt;
            public string swr;
            public string band;
        };

        PaStatus m_paStatus = new PaStatus();
        Queue rs232TX_queue = new Queue();

        public Form1()
        {
            InitializeComponent();
            pBarPwr.Minimum = 0;
            pBarPwr.Maximum = 1500;    
           
        }
        
        private void timer_start()
        {   //Timer for HRD request
            timer_HRDcom = new System.Windows.Forms.Timer();
            timer_HRDcom.Interval = 1000;
            timer_HRDcom.Start();
            timer_HRDcom.Tick += new EventHandler(timer_HRDcom_handler);
            //Timer for PA status request
            timer_PAcom = new System.Windows.Forms.Timer();
            timer_PAcom.Interval = 100;
            timer_PAcom.Start();
            timer_PAcom.Tick += new EventHandler(timer_PAcom_handler);
        }
        
        private void timer_HRDcom_handler(object sender, System.EventArgs e)
        {
            if (sender == timer_HRDcom)
            {
                thread_HRDcom = new Thread(new ThreadStart(HRD_getData));
                thread_HRDcom.IsBackground = true;
                thread_HRDcom.Start();

                thread_PAcom = new Thread(new ThreadStart(PA_getStatus));
                thread_PAcom.IsBackground = true;
                thread_PAcom.Start();
            }
        }

        private void timer_PAcom_handler(object sender, System.EventArgs e)
        {
            if (sender == timer_PAcom)
            {
               
                if (rs232TX_queue.Count > 0)
                {
                    serialSend("");
                    
                }

                updateUI();         //Update UI elements
            }
        }

        private void setupToolStripMenuItem_Click(object sender, EventArgs e)
        {   //Show program setup form
            Form_setup frmSetup = new Form_setup();
            frmSetup.ShowDialog();
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {   //Close application
            Application.Exit();
        }

        private void Form_main_Load(object sender, EventArgs e)
        {
            //read from ini file and set parameters
            IniFile ini = new IniFile(iniPath);
            paComPort = ini.IniReadValue("SSPA_Remote", "ComPort");
            if (paComPort == "") paComPort = "COM1";

            hrdIP = ini.IniReadValue("SSPA_Remote", "HrdIP");
            if (hrdIP == "") hrdIP = "127.0.0.1";

            if (ini.IniReadValue("SSPA_Remote", "HrdEnable") == "1") hrdEnable = true;
            else hrdEnable = false;

            m_SerialPort = new SerialPort(paComPort, 9600, Parity.None, 8, StopBits.One);              //Create com port
            m_SerialPort.NewLine = "\r";                                                                //Set EOL
            m_SerialPort.ReadTimeout = 1000;
            m_SerialPort.DataReceived += new SerialDataReceivedEventHandler(serial_DataReceived);       //Receive event

            timer_start();

        }


        private string HRD_message(string msg)
        {     
            //Check Client availability
            using (TcpClient tcp = new TcpClient())
            {
                IAsyncResult ar = tcp.BeginConnect(hrdIP, 7809, null, null);
                System.Threading.WaitHandle wh = ar.AsyncWaitHandle;
                try
                {
                    if (!ar.AsyncWaitHandle.WaitOne(TimeSpan.FromSeconds(1), false))
                    {
                        tcp.Close();
                      //  throw new TimeoutException();
                        return "No connection to HRD!";
                    }

                    tcp.EndConnect(ar);
                }
                finally
                {
                    wh.Close();
                }
            }

           
            //Build HRD data
            int size = 0;
            uint magic_1 = 0x1234ABCD;
            uint magic_2 = 0xABCD1234;
            uint checksum = 0;              //not used

            UnicodeEncoding encoding = new UnicodeEncoding();   //convert to UTF16
            byte[] message = encoding.GetBytes(msg);

            size = message.Length + 22;
            byte[] txData = new byte[100];
           
            Buffer.BlockCopy(BitConverter.GetBytes(size), 0, txData, 0,4);          //add length to msg
            Buffer.BlockCopy(BitConverter.GetBytes(magic_1), 0, txData, 4, 4);      //add magic_1
            Buffer.BlockCopy(BitConverter.GetBytes(magic_2), 0, txData, 8, 4);      //add magic_2
            Buffer.BlockCopy(BitConverter.GetBytes(checksum), 0, txData, 12, 4);    //add checksum      
            Buffer.BlockCopy(message, 0, txData, 16, message.Length);               //add message

            tcpClient = new TcpClient();
            tcpClient.Connect(hrdIP, 7809);
            tcpClient.ReceiveTimeout = 1000;
            tcpStream = tcpClient.GetStream();
            tcpStream.Write(txData, 0, size);                                       //Send massage
                        
            byte[] buffer = new byte[maxBuffer];                
            int gelesen = tcpStream.Read(buffer, 0, maxBuffer);                     //Receive
            string rxString = encoding.GetString(buffer, 0, gelesen);
            rxString = rxString.Substring(8);                                       //remove preamble
            rxString = rxString.Remove(rxString.Length - 3);                        //remove tail

            tcpStream.Close();
            tcpClient.Close();

            return rxString;
        }

        bool HRD_getRadios()
        {
            try
            {
                hrdRadioID = HRD_message("get radios");
                hrdRadioID = hrdRadioID.Remove(hrdRadioID.IndexOf(':'));
                return false;
            }
            catch (Exception exp)
            {
                set_StausMsg("HRD_getFrequency connection failed!");
                return true;
            }
        }

        int HRD_getFrequency()
        {
            string temp;
            try
            {
                temp = HRD_message('[' + hrdRadioID + ']' + ' ' + "get frequencies");
                // temp = HRD_message("[31] get frequencies");
                temp = temp.Remove(temp.IndexOf('-'));
                hrdFrequency = int.Parse(temp);
                set_StausMsg("HRD: " + hrdFrequency.ToString());
                return hrdFrequency;
            }
            catch (Exception exp)
            {
                set_StausMsg("HRD_getFrequency connection failed!");
                return 0;
            }
        }

        void HRD_getData()
        {
            if (hrdEnable)
            {
                if (!HRD_getRadios()) ;                    //Get radio ID
                {
                    if (HRD_getFrequency() > 0)
                    {
                        hrdFrequency = HRD_getFrequency();  //Get current VFO A frequency
                    }
                }
            }

            
            
            

        }

        //Open the serial port
         bool serialOpen()
        {

            try
            {
                if ((m_SerialPort != null))
                {
                    if (m_SerialPort.IsOpen)
                    {
                        m_SerialPort.Close();
                    }
                }
                
                m_SerialPort.Open();
                if (!m_SerialPort.IsOpen)
                {
                    timer_PAcom.Stop();
                    MessageBox.Show(string.Concat(paComPort, " failed to open"));
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {   
                timer_PAcom.Stop();
                MessageBox.Show(ex.Message);
                return true;
            }

          //  m_SerialPort.RtsEnable = true; //Set RTS to HIGH
        }
     

        void serialSend(string txd)
        {
            if ( !serialOpen())
            {
                while (rs232TX_queue.Count > 0)
                {
                    string txt = (string) rs232TX_queue.Dequeue();
                    m_SerialPort.Write(txt);
                    m_SerialPort.Write("\r");
                    Thread.Sleep(50);
                }
            }
        }

        void serial_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                serialRCV = m_SerialPort.ReadLine();
                serial_Decode();
            }
            catch (Exception ex)
            {

            }
        }

        void serial_Decode()
        {   //Decode received data into a array of strings
            string[] temp = serialRCV.Split(';');
            m_paStatus.status = int.Parse(temp[0]);
            m_paStatus.antsw = int.Parse(temp[1]);
            m_paStatus.temp1 = temp[2];
            m_paStatus.temp2 = temp[3];
            m_paStatus.voltage = temp[4];
            m_paStatus.current = temp[5];
            m_paStatus.powerDC = temp[6];
            m_paStatus.eta = temp[7];
            m_paStatus.powerAnt = temp[8];
            m_paStatus.swr = temp[9];
            m_paStatus.band = temp[10];
        }


        void updateUI()
        {   //Update the UI controls after data has been received
            if (m_paStatus.status == 0)
            {   //Disable buttons if PA is in Startup Mode
                this.btnEstop.Enabled = false;
                this.btnActive.Enabled = false; 
                this.btnAnt1.Enabled = false;   
                this.btnAnt2.Enabled = false;
                this.btnAnt3.Enabled = false;
                
            }
            else
            {
                this.btnEstop.Enabled = true;
                this.btnActive.Enabled = true;
                this.btnAnt1.Enabled = true;
                this.btnAnt2.Enabled = true;
                this.btnAnt3.Enabled = true;
            }

            if (m_paStatus.status == 1 || m_paStatus.status == 2)
            {
               btnEstop.ForeColor = Color.Yellow;
               btnEstop.BackColor = Color.OrangeRed;
               btnEstop.Text = "Emergency OFF";
            }
            else if (m_paStatus.status == 3)
            {
                btnEstop.ForeColor = Color.Blue;
                btnEstop.BackColor = Color.LightGray;
                btnEstop.Text = "ON";
            }

            if (m_paStatus.status == 2)
            {
                btnActive.ForeColor = Color.OrangeRed;
                btnActive.Text = "STANDBY";
            }

            else if (m_paStatus.status == 1 || m_paStatus.status == 3)
            { 
                btnActive.ForeColor = Color.Blue;
                btnActive.Text = "ACTIVE";
            }

            switch (m_paStatus.antsw)
            {
                case 0: 
                    btnAnt1.ForeColor = Color.Red;
                    btnAnt2.ForeColor = Color.Blue;
                    btnAnt3.ForeColor = Color.Blue;
                    break;

                case 1:
                    btnAnt1.ForeColor = Color.Blue;
                    btnAnt2.ForeColor = Color.Red;
                    btnAnt3.ForeColor = Color.Blue;
                    break;

                case 2:
                    btnAnt1.ForeColor = Color.Blue;
                    btnAnt2.ForeColor = Color.Blue;
                    btnAnt3.ForeColor = Color.Red;
                    break;
            }
            //Update measurements
            textBoxVoltage.Text = m_paStatus.voltage + " V";
            textBoxCurrent.Text = m_paStatus.current + " A";
            textBoxPowerDC.Text = m_paStatus.powerDC + " W";
            textBoxEta.Text         = m_paStatus.eta + " %";
            textBoxPowerAnt.Text = m_paStatus.powerAnt + " W";
            textBoxSWR.Text = m_paStatus.swr;
            textBoxTemp1.Text = m_paStatus.temp1 + " °C";
            textBoxTemp2.Text = m_paStatus.temp2 + " °C";
            textBoxBand.Text = m_paStatus.band;
            
            try { pBarPwr.Value = Int32.Parse(m_paStatus.powerAnt); }
            catch (Exception ex) { pBarPwr.Value = 0; }

        }

        void PA_getStatus()
        {   //Request PA status
                        
            rs232TX_queue.Enqueue("SF" + hrdFrequency.ToString());
            rs232TX_queue.Enqueue("RS");
            
        }

        private void set_StausMsg(string msg)
        {
          //  textBox_Status.AppendText(msg);

            if (InvokeRequired)
            {
                this.Invoke(new Action<string>(set_StausMsg), new object[] { msg });
                return;
            }
            textBox_Status.Text = msg;
        }

        private void button1_Click(object sender, EventArgs e)
        {
          //  serialSend("SA1");
            rs232TX_queue.Enqueue("SA1");
        }

        private void button2_Click(object sender, EventArgs e)
        {
         ///   serialSend("SA2");
            rs232TX_queue.Enqueue("SA2");
        }

        private void btnEstop_Click(object sender, EventArgs e)
        {
            if (m_paStatus.status == 1 || m_paStatus.status == 2) rs232TX_queue.Enqueue("SM0");
            else rs232TX_queue.Enqueue("SM1");
        }

        private void btnActive_Click(object sender, EventArgs e)
        {
            if (m_paStatus.status == 1) rs232TX_queue.Enqueue("SM3"); //serialSend("SM3");
            if (m_paStatus.status == 2) rs232TX_queue.Enqueue("SM2"); //serialSend("SM2");
        }

      

        private void textBoxBand_MouseDown(object sender, MouseEventArgs e)
        {
            Form_bandSelect frmBandSel = new Form_bandSelect();
            frmBandSel.Show();
         //   frmBandSel.Location = this.Location;
            frmBandSel.Location = new Point(this.Location.X + 50, this.Location.Y + 50);
        }

      
        private void button3_Click(object sender, EventArgs e)
        {
            rs232TX_queue.Enqueue("SA3");
        }
    }
}
